unit UFormCheck;

{$mode delphi}

interface

uses
  Classes, SysUtils, LazFileUtils, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, ComCtrls, Menus, ActnList, UAcronym, URegistry, Registry, UCommon,
  fgl;

type
  TReportType = (rtNone, rtNote, rtWarning, rtError);

  TReportItem = class
    Message: string;
    Position: TPoint;
    Kind: TReportType;
  end;

  { TReportItems }

  TReportItems = class(TFPGObjectList<TReportItem>)
    function AddNew(Message: string; Position: TPoint;
      Kind: TReportType = rtNone): TReportItem;
    procedure SaveToCsv(FileName: string);
  end;

  { TFormCheck }

  TFormCheck = class(TForm)
    AGoToLocation: TAction;
    ASaveToCsv: TAction;
    ActionList1: TActionList;
    ButtonLoadFromFile: TButton;
    ButtonAcronymsContent: TButton;
    ButtonAcronymsSummary: TButton;
    ButtonCheck: TButton;
    CheckBoxCaseSensitive: TCheckBox;
    EditSummaryStart: TEdit;
    EditSummaryEnd: TEdit;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    LabelAcronymCountContent: TLabel;
    LabelAcronymCountSummary: TLabel;
    ListViewReport: TListView;
    MemoDocument: TMemo;
    MenuItem1: TMenuItem;
    MenuItemGoTo: TMenuItem;
    OpenDialog1: TOpenDialog;
    PageControl1: TPageControl;
    Panel1: TPanel;
    Panel2: TPanel;
    PopupMenuReport: TPopupMenu;
    SaveDialog1: TSaveDialog;
    Splitter1: TSplitter;
    TabSheetSource: TTabSheet;
    TabSheetReport: TTabSheet;
    procedure AGoToLocationExecute(Sender: TObject);
    procedure ASaveToCsvExecute(Sender: TObject);
    procedure ButtonAcronymsSummaryClick(Sender: TObject);
    procedure ButtonAcronymsContentClick(Sender: TObject);
    procedure ButtonCheckClick(Sender: TObject);
    procedure ButtonLoadFromFileClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ListViewReportData(Sender: TObject; Item: TListItem);
    procedure MenuItemGoToClick(Sender: TObject);
  private
    AcronymDbSummary: TAcronymDb;
    AcronymDbContent: TAcronymDb;
    LastDocumentFileName: string;
    function SearchLine(Lines: TStrings; Text: string; Start: Integer = 0): Integer;
    function SearchLineReverse(Lines: TStrings; Text: string; Start: Integer = -1): Integer;
    procedure FindInSummary;
    procedure FindInContent;
    function AllowedSideChar(Before, After: Char): Boolean;
    function ParseMeaning(Acronym, Text: string; StartIndex: Integer;
      out Meaning: string; DashSeparator: Boolean = False): Boolean;
    function IsUppercaseAlpha(Text: string): Boolean;
    function IsLowercaseAlpha(Text: string): Boolean;
    function IsAlpha(Text: string): Boolean;
    function IsAcronym(Text: string): Boolean;
    function IsDigit(Text: Char): Boolean;
    procedure ReportDifferencies;
    function WordContainsLetters(Text, Letters: string): Boolean;
    function StringEqual(Text1, Text2: string): Boolean;
    procedure ReloadReport;
  public
    ReportItems: TReportItems;
    procedure UpdateInterface;
    procedure LoadConfig;
    procedure SaveConfig;
  end;

var
  FormCheck: TFormCheck;

const
  ReportTypeString: array[TReportType] of string = ('', 'Note', 'Warning', 'Error');


implementation

{$R *.lfm}

uses
  UFormAcronyms, UCore;

resourcestring
  SAcronymCountContent = 'Content acronym count:';
  SAcronymCountSummary = 'Summary acronym count:';
  SDuplicateAcronymContent = 'Duplicate acronym %s with "%s" in document body.';
  SDuplicateAcronymSummary = 'Duplicate acronym %s with "%s" in acronym summary.';
  SMissingAcronymContent = 'Document body acronym %s with meaning "%s" missing from acronym summary.';
  SMissingAcronymSummary = 'Summary acronym %s with meaning "%s" missing from document body.';
  SAcronymContentMultipleMeanings = 'Content acronym %s has multiple meanings: %s.';
  SAcronymSummaryMultipleMeanings = 'Summary acronym %s has multiple meanings: %s.';
  SAcronymWithDifferentMeaning = 'Acronym %s has different meaning for content "%s" and for summary "%s".';
  SAcronymWithDifferentMeaningCount = 'Acronym %s has different meaning count for content (%d) and for summary (%d).';
  SPluralAcronym = 'Acronym %s is defined as plural in document body.';
  SPluralAcronymUsed = 'Acronym %s is used as plural in document body.';
  SSummaryAcronyms = 'Summary acronyms';
  SContentAcronyms = 'Content acronyms';
  SAcronymUsedBeforeDefined = 'Acronym %s used before it was defined.';
  SCSVFilter = 'CSV file (.csv)|*.csv|Any file|*.*';

const
  MinAcronymLength = 2;

{ TReportItems }

function TReportItems.AddNew(Message: string; Position: TPoint;
  Kind: TReportType = rtNone): TReportItem;
begin
  Result := TReportItem.Create;
  Result.Message := Message;
  Result.Position := Position;
  Result.Kind := Kind;
  Add(Result);
end;

procedure TReportItems.SaveToCsv(FileName: string);
var
  I: Integer;
  F: TStringList;
  Line: TStringList;
begin
  F := TStringList.Create;
  Line := TStringList.Create;
  Line.StrictDelimiter := True;
  try
    Line.Clear;
    for I := 0 to Count - 1 do
    with TReportItem(Items[I]) do begin
      Line.Clear;
      if Position <> Point(0, 0) then
        Line.Add(IntToStr(Position.X) + ', ' + IntToStr(Position.Y))
        else Line.Add('');
      Line.Add(ReportTypeString[Kind]);
      Line.Add(Message);
      F.Add(Line.CommaText);
    end;
    F.SaveToFile(FileName);
  finally
    F.Free;
    Line.Free;
  end;
end;

{ TFormCheck }

procedure TFormCheck.ButtonCheckClick(Sender: TObject);
begin
  ReportItems.Clear;
  UpdateInterface;
  FindInSummary;
  FindInContent;
  ReportDifferencies;
  UpdateInterface;
  ReloadReport;
  TabSheetReport.Show;
end;

procedure TFormCheck.ButtonLoadFromFileClick(Sender: TObject);
begin
  OpenDialog1.InitialDir := ExtractFileDir(LastDocumentFileName);
  OpenDialog1.FileName := ExtractFileName(LastDocumentFileName);
  if OpenDialog1.Execute then begin
    LastDocumentFileName := OpenDialog1.FileName;
    MemoDocument.Lines.LoadFromFile(OpenDialog1.FileName);
    TabSheetSource.Show;
  end;
end;

procedure TFormCheck.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  Core.PersistentForm1.Save(Self);
end;

procedure TFormCheck.FormCreate(Sender: TObject);
begin
  AcronymDbSummary := TAcronymDb.Create;
  AcronymDbContent := TAcronymDb.Create;
  Core.Translator.TranslateComponentRecursive(Self);
  Core.ThemeManager.UseTheme(Self);
  ReportItems := TReportItems.Create;
end;

procedure TFormCheck.FormDestroy(Sender: TObject);
begin
  ReportItems.Free;
  AcronymDbSummary.Free;
  AcronymDbContent.Free;
end;

procedure TFormCheck.FormShow(Sender: TObject);
begin
  PageControl1.TabIndex := 0;
  Core.PersistentForm1.Load(Self);
  if FileExists(LastDocumentFileName) then
    MemoDocument.Lines.LoadFromFile(LastDocumentFileName);
  UpdateInterface;
end;

procedure TFormCheck.ListViewReportData(Sender: TObject; Item: TListItem);
begin
  if Item.Index < ReportItems.Count then
    with ReportItems[Item.Index] do begin
      if Position <> Point(0, 0) then
        Item.Caption := IntToStr(Position.X) + ', ' + IntToStr(Position.Y)
        else Item.Caption := '';
      Item.Data := ReportItems[Item.Index];
      Item.SubItems.Add(ReportTypeString[Kind]);
      Item.SubItems.Add(Message);
    end;
end;

procedure TFormCheck.MenuItemGoToClick(Sender: TObject);
begin

end;

procedure TFormCheck.ButtonAcronymsContentClick(Sender: TObject);
var
  FormAcronyms: TFormAcronyms;
begin
  FormAcronyms := TFormAcronyms.Create(Self);
  try
    FormAcronyms.Acronyms := AcronymDbContent.Acronyms;
    FormAcronyms.Caption := SContentAcronyms;
    FormAcronyms.ShowModal;
  finally
    FreeAndNil(FormAcronyms);
  end;
end;

procedure TFormCheck.ButtonAcronymsSummaryClick(Sender: TObject);
var
  FormAcronyms: TFormAcronyms;
begin
  FormAcronyms := TFormAcronyms.Create(Self);
  try
    FormAcronyms.Acronyms := AcronymDbSummary.Acronyms;
    FormAcronyms.Caption := SSummaryAcronyms;
    FormAcronyms.ShowModal;
  finally
    FreeAndNil(FormAcronyms);
  end;
end;

procedure TFormCheck.ASaveToCsvExecute(Sender: TObject);
begin
  SaveDialog1.InitialDir := ExtractFileDir(LastDocumentFileName);
  SaveDialog1.DefaultExt := '.csv';
  SaveDialog1.FileName := ExtractFileNameWithoutExt(ExtractFileName(LastDocumentFileName)) + SaveDialog1.DefaultExt;
  SaveDialog1.Filter := SCSVFilter;
  if SaveDialog1.Execute then begin
    ReportItems.SaveToCsv(SaveDialog1.FileName);
  end;
end;

procedure TFormCheck.AGoToLocationExecute(Sender: TObject);
begin
  if Assigned(ListViewReport.Selected) then
  with TReportItem(ListViewReport.Selected.Data) do
  if Position <> Point(0, 0) then begin
    MemoDocument.CaretPos := Position;
    TabSheetSource.Show;
  end;
end;

function TFormCheck.SearchLine(Lines: TStrings; Text: string; Start: Integer = 0): Integer;
begin
  Result := Start;
  while (Result < Lines.Count) and (Pos(Text, Lines[Result]) = 0) do Inc(Result);
  if Result >= Lines.Count then Result := -1;
end;

function TFormCheck.SearchLineReverse(Lines: TStrings; Text: string;
  Start: Integer = -1): Integer;
begin
  if Start = -1 then Result := Lines.Count - 1
    else Result := Start;
  while (Result >= 0) and (Pos(Text, Lines[Result]) = 0) do Dec(Result);
end;

procedure TFormCheck.FindInSummary;
var
  AcronymSectionStart: Integer;
  AcronymSectionEnd: Integer;
  I: Integer;
  Line: string;
  Acronym: string;
  Meaning: string;
  Index: Integer;
begin
  AcronymDbSummary.Acronyms.Clear;

  AcronymSectionStart := SearchLineReverse(MemoDocument.Lines, EditSummaryStart.Text);
  if AcronymSectionStart <> -1 then begin
    AcronymSectionEnd := SearchLine(MemoDocument.Lines, EditSummaryEnd.Text, AcronymSectionStart + 1);
    if AcronymSectionEnd <> -1 then begin
      Acronym := '';
      for I := AcronymSectionStart + 1 to AcronymSectionEnd - 1 do begin
        Line := Trim(MemoDocument.Lines[I]);
        Line := StringReplace(Line, #9, ' ', [rfReplaceAll]);
        if Line <> '' then begin
          if (Acronym <> '') and IsUppercaseAlpha(Acronym) then begin
            Meaning := Line;
          end else begin
            Index := Pos(' ', Line);
            if Index > 0 then begin
              Acronym := Copy(Line, 1, Index - 1);
              Meaning := Trim(Copy(Line, Index + 1, Length(Line)));
            end else begin
              if Acronym = '' then Acronym := Line
                else Meaning := Line;
            end;
          end;
          if (Acronym <> '') and IsUppercaseAlpha(Acronym) and (Meaning <> '') then begin
            if Assigned(AcronymDbSummary.SearchAcronym(Acronym, Meaning)) then
              ReportItems.AddNew(Format(SDuplicateAcronymSummary, [Acronym, Meaning]), Point(0, I), rtWarning)
              else AcronymDbSummary.AddAcronym(Acronym, Meaning);
            Acronym := '';
            Meaning := '';
          end;
        end;
      end;
    end;
  end;
end;

procedure TFormCheck.FindInContent;
var
  Text: string;
  Index: Integer;
  State: (stNone, stAcronymUsage, stAcronymDefinition);
  Acronym: string;
  AcronymCharBefore: Char;
  AcronymCharAfter: Char;
  Lines: TStringList;
  HasUpperCase: Boolean;
  HasLowerCase: Boolean;
  I: Integer;
  J: Integer;
  Line: string;
  Meaning: string;
  Meaning1: string;
  Meaning2: string;
  HasMeaning1: Boolean;
  HasMeaning2: Boolean;
  Plural: Boolean;
  StartIndex: Integer;
  Acro: TAcronym;
begin
  AcronymDbContent.Acronyms.Clear;

  // Make lowercase lines where all alpha characters are in uppercase
  // These are usually first level chapter titles or first page text
  Lines := TStringList.Create;
  Lines.Assign(MemoDocument.Lines);
  for I := 0 to Lines.Count - 1 do begin
    HasLowerCase := False;
    HasUpperCase := False;
    Line := Lines[I];
    for J := 1 to Length(Line) do begin
      if IsUppercaseAlpha(Line[J]) then HasUpperCase := True;
      if IsLowercaseAlpha(Line[J]) then HasLowerCase := True;
    end;
    if HasUpperCase and not HasLowerCase then
      Lines[I] := LowerCase(Lines[I]);
  end;

  // Find acronyms usage in text
  Text := Lines.Text;
  Text := StringReplace(Text, #9, ' ', [rfReplaceAll]);
  Text := StringReplace(Text, LineEnding, ' ', [rfReplaceAll]);
  Index := 1;
  AcronymCharBefore := ' ';
  AcronymCharAfter := ' ';
  State := stNone;
  StartIndex := 0;
  Acronym := '';
  repeat
    if State = stAcronymUsage then begin
      if not IsUppercaseAlpha(Text[Index]) then begin
        if Text[Index] = 's' then begin
          Acronym := Acronym + Text[Index];
          Inc(Index);
        end;
        if (Index) < Length(Text) then AcronymCharAfter := Text[Index]
          else AcronymCharAfter := ' ';
        State := stNone;

        // Allow plural acronyms with ending 's' character
        if (Length(Acronym) >= 1) and (Acronym[Length(Acronym)] = 's') then begin
          Acronym := Copy(Acronym, 1, Length(Acronym) - 1);
          Plural := True;
        end else Plural := False;

        // Acronyms should not contain numbers
        if (Length(Acronym) >= MinAcronymLength) and
          AllowedSideChar(AcronymCharBefore, AcronymCharAfter) then begin
            // If plural acronym then try to remove ending 's' character from the meaning
            if Plural then ReportItems.AddNew(Format(SPluralAcronymUsed, [Acronym]), Point(0, 0), rtNote);

            Acro := AcronymDbContent.Acronyms.SearchByName(Acronym);
            if not Assigned(Acro) then begin
              ReportItems.AddNew(Format(SAcronymUsedBeforeDefined, [Acronym]), Point(0, 0), rtNote);
              AcronymDbContent.AddAcronym(Acronym, '');
            end;
        end;
      end else Acronym := Acronym + Text[Index];
    end else
    if State = stAcronymDefinition then begin
      if Text[Index] = ')' then begin
        // Allow plural acronyms with ending 's' character
        if (Length(Acronym) >= 1) and (Acronym[Length(Acronym)] = 's') then begin
          Acronym := Copy(Acronym, 1, Length(Acronym) - 1);
          Plural := True;
        end else Plural := False;
        if IsAcronym(Acronym) then begin
          HasMeaning1 := ParseMeaning(Acronym, Text, StartIndex - 1, Meaning1);
          if HasMeaning1 then Meaning := Meaning1;
          HasMeaning2 := ParseMeaning(Acronym, Text, StartIndex - 1, Meaning2, True);
          if HasMeaning2 then Meaning := Meaning2;
          if HasMeaning1 and HasMeaning2 then begin
            if Length(Meaning1) > Length(Meaning2) then Meaning := Meaning1
              else Meaning := Meaning2;
          end;
          if HasMeaning1 or HasMeaning2 then begin
            // If plural acronym then try to remove ending 's' character from the meaning
            if Plural then ReportItems.AddNew(Format(SPluralAcronym, [Acronym]), Point(0, 0), rtNote);
            if Plural and (Length(Meaning) >= 1) and (Meaning[Length(Meaning)] = 's') then begin
              Meaning := Copy(Meaning, 1, Length(Meaning) - 1);
            end;
            if Assigned(AcronymDbContent.SearchAcronym(Acronym, Meaning)) then
              ReportItems.AddNew(Format(SDuplicateAcronymContent, [Acronym, Meaning]), Point(0, 0), rtWarning)
              else AcronymDbContent.AddAcronym(Acronym, Meaning);
          end;
        end else
          // No acronym inside parenthesis, continue with parsing inside
          Index := StartIndex + 1;
        State := stNone;
      end else begin
        Acronym := Acronym + Text[Index];
      end;
    end else begin
      if Text[Index] = '(' then begin
        State := stAcronymDefinition;
        Acronym := '';
        StartIndex := Index;
      end else
      if IsUppercaseAlpha(Text[Index]) then begin
        State := stAcronymUsage;
        Acronym := Text[Index];
        if (Index - 1) >= 1 then AcronymCharBefore := Text[Index - 1]
          else AcronymCharBefore := ' ';
      end;
    end;
    Inc(Index);
  until Index > Length(Text);
  Lines.Free;
end;

function TFormCheck.AllowedSideChar(Before, After: Char): Boolean;
begin
  Result := ((Before = ' ') or (Before = #10) or (Before = #13) or (Before = ',') or
    (Before = ';') or (Before = '(') or (Before = ')'))
    and ((After = ' ') or (After = #10) or (After = #13) or (After = ',') or
    (After = '.') or (After = ';') or (After = '(') or (After = ')'));
end;

function TFormCheck.ParseMeaning(Acronym, Text: string; StartIndex: Integer;
  out Meaning: string; DashSeparator: Boolean): Boolean;
var
  StartIndex2: Integer;
  StartIndex3: Integer;
  StartIndex4: Integer;
  LetterIndex: Integer;
  OneWord: string;
  WordLetterIndex: Integer;
  WordCount: Integer;
  WordCountWrong: Integer;
begin
  Result := True;
  Meaning := '';
  StartIndex2 := StartIndex;
  LetterIndex := Length(Acronym);
  WordCount := 0;
  WordCountWrong := 0;
  while Length(Acronym) > 0 do begin
    StartIndex3 := PosFromIndexReverse(' ', Text, StartIndex2);
    if DashSeparator then begin
      StartIndex4 := PosFromIndexReverse('-', Text, StartIndex2);
      if StartIndex4 > StartIndex3 then StartIndex3 := StartIndex4;
    end;

    if StartIndex3 = 0 then Break;
    OneWord := Copy(Text, StartIndex3 + 1, StartIndex2 - StartIndex3);
    if OneWord = '$' then begin
      // Avoid parsing Bash variables
      Result := False;
      Exit;
    end;
    if Trim(OneWord) = '' then begin
      StartIndex2 := StartIndex3 - 1;
      Continue;
    end;
    // Is first letter capital?
    if (Length(OneWord) > 0) and IsAlpha(OneWord[1]) then begin
      WordLetterIndex := PosFromIndexReverse(LowerCase(OneWord[1]), LowerCase(Copy(Acronym, 1, LetterIndex)), LetterIndex);
      if WordLetterIndex > 0 then begin
        // First letter was found in acronym
        if WordLetterIndex <= LetterIndex then begin
          if not WordContainsLetters(LowerCase(OneWord), LowerCase(Copy(Acronym, WordLetterIndex, LetterIndex - WordLetterIndex + 1))) then begin
            Result := False;
            Exit;
          end;
          LetterIndex := WordLetterIndex - 1;
        end else begin
          Dec(LetterIndex);
        end;
        WordCountWrong := 0;
      end else begin
        Inc(WordCountWrong);
        if WordCountWrong > 1 then begin
          Result := False;
          Exit;
        end;
      end;
    end else begin
      Inc(WordCountWrong);
      if WordCountWrong > 1 then begin
        Result := False;
        Exit;
      end;
    end;
    StartIndex2 := StartIndex3 - 1;
    if LetterIndex < 1 then Break;
    Inc(WordCount);
    if WordCount > 2 * Length(Acronym) then begin
      // False acronym in braces with too much words
      Result := False;
      Exit;
    end;
  end;
  Meaning := Trim(Copy(Text, StartIndex2 + 1, StartIndex - StartIndex2));
end;

function TFormCheck.IsUppercaseAlpha(Text: string): Boolean;
var
  I: Integer;
begin
  I := 1;
  Result := True;
  while (I <= Length(Text)) do begin
    if not (Text[I] in ['A'..'Z']) then begin
      Result := False;
      Break;
    end;
    Inc(I);
  end;
end;

function TFormCheck.IsLowercaseAlpha(Text: string): Boolean;
var
  I: Integer;
begin
  I := 1;
  Result := True;
  while (I <= Length(Text)) do begin
    if not (Text[I] in ['a'..'z']) then begin
      Result := False;
      Break;
    end;
    Inc(I);
  end;
end;

function TFormCheck.IsAlpha(Text: string): Boolean;
var
  I: Integer;
begin
  I := 1;
  Result := True;
  while (I <= Length(Text)) do begin
    if not (Text[I] in ['A'..'Z']) and not (Text[I] in ['a'..'z']) then begin
      Result := False;
      Break;
    end;
    Inc(I);
  end;
end;

function TFormCheck.IsAcronym(Text: string): Boolean;
const
  MinAcronymLength = 2;
begin
  Result := (Length(Text) >= MinAcronymLength) and IsUppercaseAlpha(Text);
end;

function TFormCheck.IsDigit(Text: Char): Boolean;
begin
  Result := Text in ['0'..'9'];
end;

procedure TFormCheck.ReportDifferencies;
var
  I: Integer;
  J: Integer;
  Acronym: TAcronym;
  Acronym2: TAcronym;
  Meaning: TAcronymMeaning;
  Meaning2: TAcronymMeaning;
begin
  // In content but not in summary
  for I := 0 to AcronymDbContent.Acronyms.Count - 1 do begin
    Acronym := TAcronym(AcronymDbContent.Acronyms[I]);
    if Acronym.Meanings.Count > 1 then
      ReportItems.AddNew(Format(SAcronymContentMultipleMeanings, [Acronym.Name,
        Acronym.Meanings.GetNames]), Point(0, 0), rtWarning);
    Acronym2 := AcronymDbSummary.Acronyms.SearchByName(Acronym.Name);
    if not Assigned(Acronym2) then
    for J := 0 to Acronym.Meanings.Count - 1 do begin
      Meaning := TAcronymMeaning(Acronym.Meanings[J]);
      if not Assigned(AcronymDbSummary.SearchAcronym(Acronym.Name, Meaning.Name, [sfCaseInsensitive])) then
        ReportItems.AddNew(Format(SMissingAcronymContent, [Acronym.Name, Meaning.Name]), Point(0, 0), rtWarning);
    end;
  end;

  // In summary but not in content
  for I := 0 to AcronymDbSummary.Acronyms.Count - 1 do begin
    Acronym := TAcronym(AcronymDbSummary.Acronyms[I]);
    if Acronym.Meanings.Count > 1 then
      ReportItems.AddNew(Format(SAcronymSummaryMultipleMeanings, [Acronym.Name, Acronym.Meanings.GetNames]), Point(0, 0), rtWarning);
    Acronym2 := AcronymDbContent.Acronyms.SearchByName(Acronym.Name);
    if not Assigned(Acronym2) then
    for J := 0 to Acronym.Meanings.Count - 1 do begin
      Meaning := TAcronymMeaning(Acronym.Meanings[J]);
      if not Assigned(AcronymDbContent.SearchAcronym(Acronym.Name, Meaning.Name, [sfCaseInsensitive])) then
        ReportItems.AddNew(Format(SMissingAcronymSummary, [Acronym.Name, Meaning.Name]), Point(0, 0), rtWarning);
    end;
  end;

  // With different meaning
  for I := 0 to AcronymDbSummary.Acronyms.Count - 1 do begin
    Acronym := TAcronym(AcronymDbSummary.Acronyms[I]);
    Acronym2 := AcronymDbContent.Acronyms.SearchByName(Acronym.Name);
    if Assigned(Acronym2) then begin
      if (Acronym.Meanings.Count = 1) and (Acronym2.Meanings.Count = 1) then begin
        Meaning := TAcronymMeaning(Acronym.Meanings[0]);
        Meaning2 := TAcronymMeaning(Acronym2.Meanings[0]);
        if not StringEqual(Meaning.Name, Meaning2.Name) then
          ReportItems.AddNew(Format(SAcronymWithDifferentMeaning, [Acronym.Name, Meaning2.Name, Meaning.Name]), Point(0, 0), rtWarning);
      end else
        ReportItems.AddNew(Format(SAcronymWithDifferentMeaningCount, [Acronym.Name, Acronym2.Meanings.Count, Acronym.Meanings.Count]), Point(0, 0), rtWarning);
    end;
  end;
end;

function TFormCheck.WordContainsLetters(Text, Letters: string): Boolean;
var
  I: Integer;
  LetterIndex: Integer;
begin
  Result := True;
  for I := 1 to Length(Letters) do begin
    LetterIndex := Pos(Letters[I], Text);
    if LetterIndex > 0 then begin
      Text := Copy(Text, LetterIndex + 1, Length(Text));
    end else begin
      Result := False;
      Break;
    end;
  end;
end;

function TFormCheck.StringEqual(Text1, Text2: string): Boolean;
begin
  if CheckBoxCaseSensitive.Checked then Result := Text1 = Text2
    else Result := LowerCase(Text1) = LowerCase(Text2);
end;

procedure TFormCheck.ReloadReport;
begin
  ListViewReport.Items.Count := ReportItems.Count;
  ListViewReport.Refresh;
end;

procedure TFormCheck.UpdateInterface;
begin
  LabelAcronymCountContent.Caption := SAcronymCountContent + ' ' + IntToStr(AcronymDbContent.GetMeaningsCount);
  LabelAcronymCountSummary.Caption := SAcronymCountSummary + ' ' + IntToStr(AcronymDbSummary.GetMeaningsCount);
end;

procedure TFormCheck.LoadConfig;
begin
  with TRegistryEx.Create do
  try
    RootKey := RegistryRootHKEY[Core.ApplicationInfo1.RegistryRoot];
    OpenKey(Core.ApplicationInfo1.RegistryKey, True);
    EditSummaryStart.Text := ReadStringWithDefault('SummaryStart', 'ACRONYMS AND ABBREVIATIONS');
    EditSummaryEnd.Text := ReadStringWithDefault('SummaryEnd', 'Appendix');
    LastDocumentFileName := ReadStringWithDefault('LastDocumentFileName', '');
    CheckBoxCaseSensitive.Checked := ReadBoolWithDefault('CaseSensitiveComparison', False);
  finally
    Free;
  end;
end;

procedure TFormCheck.SaveConfig;
begin
  with TRegistryEx.Create do
  try
    RootKey := RegistryRootHKEY[Core.ApplicationInfo1.RegistryRoot];
    OpenKey(Core.ApplicationInfo1.RegistryKey, True);
    WriteString('SummaryStart', EditSummaryStart.Text);
    WriteString('SummaryEnd', EditSummaryEnd.Text);
    WriteString('LastDocumentFileName', LastDocumentFileName);
    WriteBool('CaseSensitiveComparison', CheckBoxCaseSensitive.Checked);
  finally
    Free;
  end;
end;

end.

